library("tidyverse")

NICKNAME_TO_FINGERPRINT <- c(
	"flakey" = "5481936581E23D2D178105D44DB6915AB06BFB7F",
	"crusty" = "91DA221A149007D0FD9E5515F5786C3DD07E4BB0"
)
FINGERPRINT_TO_SITE <- c(
	"5481936581E23D2D178105D44DB6915AB06BFB7F" = "snowflake-01",
	"91DA221A149007D0FD9E5515F5786C3DD07E4BB0" = "snowflake-02"
)

COL_NAMES <- c("date", "transport", "nickname", "dst", "instance", "family", "num_sockets")
socketcount <- bind_rows(
	read_csv("socketcount.snowflake-01.csv.gz", col_names = COL_NAMES),
	read_csv("socketcount.snowflake-02.csv.gz", col_names = COL_NAMES)
) %>%
	mutate(fingerprint = NICKNAME_TO_FINGERPRINT[nickname])

# userstats-bridge-transport-multi.csv comes from
# https://gitlab.torproject.org/tpo/network-health/metrics/timeline
userstats <- read_csv("userstats-bridge-transport-multi.csv") %>%
	mutate(date = lubridate::as_datetime(date)) %>%
	mutate(num_users = users / (coverage / pmax(num_instances, coverage)))

options(width = 160)
userstats %>%
	filter(fingerprint == "5481936581E23D2D178105D44DB6915AB06BFB7F" & transport == "snowflake" & num_users < 20000)

data <- bind_rows(
	socketcount %>%
		filter(dst == "haproxy") %>%
		mutate(variable = "num_sockets", value = num_sockets),
	userstats %>%
		mutate(variable = "num_users", value = num_users)
) %>%
	filter(transport == "snowflake") %>%
	mutate(variable = fct_relevel(variable, "num_users", "num_sockets")) %>%
	mutate(site = FINGERPRINT_TO_SITE[fingerprint])

p <- ggplot(data) +
	geom_line(
		aes(date, value, color = variable)
	) +
	scale_x_datetime(limits = lubridate::ymd_hms(c(
		"2023-06-14 00:00:00",
		"2023-07-03 00:00:00"
	))) +
	scale_y_continuous(
		limits = c(0, NA),
		labels = scales::label_comma()
	) +
	facet_grid(rows = vars(site), scales = "free_y") +
	theme_light()
ggsave("socketcount-parallel.png", p, width = 8, height = 5)

data <- inner_join(
	socketcount %>% filter(dst == "haproxy") %>% mutate(date = lubridate::as_date(date)),
	userstats,
	by = c("date", "fingerprint", "transport")
) %>%
	filter(transport == "snowflake") %>%
	mutate(site = FINGERPRINT_TO_SITE[fingerprint])

p <- ggplot(
	data %>%
		group_by(date, site) %>%
		summarize(across(c(num_users, num_sockets), mean), .groups = "drop")
) +
	geom_abline(intercept = 0, slope = 1, alpha = 0.2) +
	geom_point(aes(num_users, num_sockets), alpha = 0.5, size = 3) +
	scale_x_continuous(
		limits = c(0, NA),
		labels = scales::label_comma()
	) +
	scale_y_continuous(
		limits = c(0, NA),
		labels = scales::label_comma()
	) +
	facet_grid(rows = vars(site)) +
	coord_fixed() +
	theme_light()
ggsave("socketcount-scatter.png", p, width = 8, height = 8)
